Type TWebParser
	
	Field Stream:TStream
	Field PageData:String
	Field Elements:TList
	
	Function Create:TWebParser()
		Local A:TWebParser = New TWebParser
		A.Elements = CreateList()
		Return( A )
	End Function
	
	Method LoadFromURL( URL:String )
		LoadFromHTML( URL )
		ParseElements()
	End Method
	
	Method LoadFromString( Str:String )
		PageData = Str
		ParseElements()
	End Method
	
	Method Clear()
		ClearList( Self.Elements )
		PageData = ""
	End Method
	
	Method LoadFromHTML( URL:String )
		Self.Stream = ReadStream( URL )
		
		Repeat
			PageData = PageData + ReadLine( Self.Stream )
		Until( Eof( Self.Stream ) )
		
		CloseStream( Self.Stream )
	End Method
	
	Method ReturnHTML:String( URL:String )
		Self.Stream = ReadStream( URL )
		
		Local Data:String
		
		Repeat
			Data = Data + ReadLine( Self.Stream )
		Until( Eof( Self.Stream ) )
		
		CloseStream( Self.Stream )
		
		Return Data
	End Method
	
	Method ParseElements()
		Local StartBracket:Int	= 0
		Local EndBracket:Int	= 0
		
		Repeat
			StartBracket = Instr( PageData, "<", StartBracket + 1 )
			If( StartBracket <> 0 )
				EndBracket = Instr( PageData, ">", StartBracket + 1 )
				
				Local BracketContent:String = Mid( PageData, StartBracket + 1, EndBracket - StartBracket - 1 )
				Local BracketType:String
				If( Instr( BracketContent, " " ) )
					BracketType = Left( BracketContent, Instr( BracketContent, " " ) - 1 )
				Else
					BracketType = BracketContent
				EndIf
				
				If( Left( BracketContent, 1 ) = "/" )
				ElseIf( Right( BracketContent, 1 ) = "/" )
				Else
					Local NextType:Int = Instr( PageData, "/" + BracketType, EndBracket + 1 )
					If( NextType )
						ListAddLast( Self.Elements, TElement.Create( BracketType, Mid( BracketContent, Len( BracketType ) + 2 ), EndBracket + 1, NextType - 2 - EndBracket ) )
					EndIf
				EndIf
			Else
				Exit
			EndIf
		Forever
	End Method
	
	Method GetElementContent:String( Name:String, Parameters:String = "NULL", Skips:Int = 0 )
		Local SkipCount:Int = 0
		For Local A:TElement = EachIn Self.Elements
			If( Lower( A.Name ) = Lower( Name ) )
				If( Parameters <> "NULL" )
					If( Lower( A.Parameters ) = Lower( Parameters ) )
						If( SkipCount = Skips )
							Return( Mid( PageData, A.ContentStart, A.ContentSize ) )
						Else
							SkipCount = SkipCount + 1
						EndIf
					EndIf
				Else
					If( SkipCount = Skips )
						Return( Mid( PageData, A.ContentStart, A.ContentSize ) )
					Else
						SkipCount = SkipCount + 1
					EndIf
				EndIf
			EndIf
		Next
	End Method
	
	Method GetElements:TList( Name:String, Parameters:String = "NULL" )
		Local List:TList = CreateList()
		For Local A:TElement = EachIn Self.Elements
			If( Lower( A.Name ) = Lower( Name ) )
				If( Parameters <> "NULL" )
					If( Lower( A.Parameters ) = Lower( Parameters ) )
						ListAddLast( List, A )
					EndIf
				Else
					ListAddLast( List, A )
				EndIf
			EndIf
		Next
		Return List
	End Method
	
	Method GetElement:TElement( Name:String, Parameters:String = "NULL" )
		For Local A:TElement = EachIn Self.Elements
			If( Lower( A.Name ) = Lower( Name ) )
				If( Parameters <> "NULL" )
					If( Lower( A.Parameters ) = Lower( Parameters ) )
						Return A
					EndIf
				Else
					Return A
				EndIf
			EndIf
		Next
	End Method
	
	Method GetNextElement:TElement( Element:TElement )
		Local Found:Int = False
		For Local A:TElement = EachIn Self.Elements
			If( Found = False )
				If( A = Element )
					Found = True
				EndIf
			Else
				Return A
			EndIf
		Next
	End Method
	
End Type